# pyright: reportIncompatibleVariableOverride=false

"""Implementations for Graphics class.
"""

from typing import List
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._display import circle as _circle
from apysc._display import ellipse as _ellipse
from apysc._display import line as _line
from apysc._display import path as _path
from apysc._display import polygon as _polyg
from apysc._display import polyline as _polyline
from apysc._display import sprite
from apysc._display import triangle as _triangle
from apysc._display.begin_fill_mixin import BeginFillMixIn
from apysc._display.child_mixin import ChildMixIn
from apysc._display.css_mixin import CssMixIn
from apysc._display.display_object import DisplayObject
from apysc._display.graphics_clear_mixin import GraphicsClearMixIn
from apysc._display.line_style_mixin import LineStyleMixIn
from apysc._display.rectangle import Rectangle
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._geom.path_data_base import PathDataBase
from apysc._geom.point2d import Point2D
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._type.array import Array
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class Graphics(
    XMixIn,
    YMixIn,
    SetOverflowVisibleSettingMixIn,
    CssMixIn,
    DisplayObject,
    BeginFillMixIn,
    LineStyleMixIn,
    GraphicsClearMixIn,
    ChildMixIn,
    InitializeWithBaseValueInterface,
):
    """
    Create an object that has each vector graphics interface.

    References
    ----------
    - Graphics
        - https://simon-ritchie.github.io/apysc/en/graphics.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50
    ... )
    >>> rectangle.x
    Number(50.0)

    >>> circle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=50)
    >>> circle.x
    Number(100.0)
    """

    _current_line: Optional["_polyline.Polyline"] = None

    @final
    @arg_validation_decos.is_display_object_container(
        arg_position_index=1, optional=False
    )
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=True)
    @arg_validation_decos.is_builtin_string(arg_position_index=3, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        parent: "sprite.Sprite",
        variable_name: Optional[str] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        Create an object that has each vector graphics interface.

        Parameters
        ----------
        parent : Sprite
            A parent instance.
        variable_name : str or None, default None
            Variable name to set. Specified only when
            a subclass instantiation.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - Graphics
            - https://simon-ritchie.github.io/apysc/en/graphics.html
        """
        from apysc._color.colorless import COLORLESS
        from apysc._display.sprite import Sprite
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._validation import display_validation

        self._variable_name_suffix = variable_name_suffix
        display_validation.validate_sprite(sprite=parent)
        self.parent_sprite: Sprite = parent
        if variable_name is None:
            variable_name = expression_variables_util.get_next_variable_name(
                type_name=var_names.GRAPHICS
            )
        super(Graphics, self).__init__(variable_name=variable_name)

        suffix: str = self._get_attr_or_variable_name_suffix(
            value_identifier="fill_color"
        )
        self._fill_color = COLORLESS

        suffix = self._get_attr_or_variable_name_suffix(value_identifier="fill_alpha")
        self._fill_alpha = Number(1.0, variable_name_suffix=suffix)

        suffix = self._get_attr_or_variable_name_suffix(value_identifier="line_color")
        self._line_color = COLORLESS

        suffix = self._get_attr_or_variable_name_suffix(value_identifier="line_alpha")
        self._line_alpha = Number(1.0, variable_name_suffix=suffix)

        suffix = self._get_attr_or_variable_name_suffix(
            value_identifier="line_thickness"
        )
        self._line_thickness = Int(1.0, variable_name_suffix=suffix)

        self._initialize_line_cap_if_not_initialized()
        self._initialize_line_joints_if_not_initialized()
        self._initialize_line_dot_setting_if_not_initialized()
        self._initialize_line_dash_setting_if_not_initialized()
        self._initialize_line_round_dot_setting_if_not_initialized()
        self._initialize_line_dash_dot_setting_if_not_initialized()

        suffix = self._get_attr_or_variable_name_suffix(value_identifier="children")
        self._children = Array([], variable_name_suffix=suffix)

        self._append_constructor_expression()
        self.parent_sprite.add_child(child=self)
        self._set_overflow_visible_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append constructor expression.
        """
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._expression import expression_data_util

        stage: Stage = get_stage()
        expression: str = f"var {self.variable_name} = {stage.variable_name}.nested();"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=3, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=3, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=4, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=4, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=5, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_rect(
        self,
        *,
        x: Union[float, Number],
        y: Union[float, Number],
        width: Union[int, Int],
        height: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> Rectangle:
        """
        Draw a rectangle vector graphics.

        Parameters
        ----------
        x : float or Number
            X position to start drawing.
        y : float or Number
            Y position to start drawing.
        width : Int or int
            Rectangle width.
        height : Int or int
            Rectangle height.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        rectangle : Rectangle
            Created rectangle.

        References
        ----------
        - Graphics draw_rect interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_rect.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.x
        Number(50.0)

        >>> rectangle.width
        Int(50)

        >>> rectangle.fill_color
        Color("#00aaff")
        """
        rectangle: Rectangle = Rectangle._create_with_graphics(
            graphics=self,
            x=x,
            y=y,
            width=width,
            height=height,
            variable_name_suffix=variable_name_suffix,
        )
        return rectangle

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=3, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=3, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=4, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=4, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=5, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=5, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=6, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=6, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=7, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_round_rect(
        self,
        *,
        x: Union[float, Number],
        y: Union[float, Number],
        width: Union[int, Int],
        height: Union[int, Int],
        ellipse_width: Union[int, Int],
        ellipse_height: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> Rectangle:
        """
        Draw a rounded rectangle vector graphics.

        Parameters
        ----------
        x : float or Number
            X-coordinate to start drawing.
        y : float or Number
            Y-coordinate to start drawing.
        width : Int or int
            Rectangle width.
        height : Int or int
            Rectangle height.
        ellipse_width : Int or int
            Ellipse width of the rectangle corner.
        ellipse_height : Int or int
            Ellipse height of the rectangle corner.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        rectangle : Rectangle
            Created rectangle.

        References
        ----------
        - Graphics draw_round_rect interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_round_rect.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> round_rect: ap.Rectangle = sprite.graphics.draw_round_rect(
        ...     x=50, y=50, width=50, height=50, ellipse_width=10, ellipse_height=15
        ... )
        >>> round_rect.ellipse_width
        Int(10)

        >>> round_rect.ellipse_height
        Int(15)
        """
        rectangle: Rectangle = Rectangle._create_with_graphics(
            graphics=self,
            x=x,
            y=y,
            width=width,
            height=height,
            variable_name_suffix=variable_name_suffix,
        )
        if isinstance(ellipse_width, int):
            ellipse_width = Int(
                ellipse_width, variable_name_suffix=variable_name_suffix
            )
        if isinstance(ellipse_height, int):
            ellipse_height = Int(
                ellipse_height, variable_name_suffix=variable_name_suffix
            )
        rectangle.ellipse_width = ellipse_width
        rectangle.ellipse_height = ellipse_height
        return rectangle

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=3, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=3, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=4, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_circle(
        self,
        *,
        x: Union[float, Number],
        y: Union[float, Number],
        radius: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> "_circle.Circle":
        """
        Draw a circle vector graphics.

        Parameters
        ----------
        x : float or Number
            X-coordinate of the circle center.
        y : float or Number
            Y-coordinate of the circle center.
        radius : Int or int
            Circle radius.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        circle : Circle
            Created circle graphics instance.

        References
        ----------
        - Graphics draw_circle interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_circle.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> circle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=50)
        >>> circle.x
        Number(100.0)

        >>> circle.y
        Number(100.0)

        >>> circle.radius
        Int(50)

        >>> circle.fill_color
        Color("#00aaff")
        """
        circle: _circle.Circle = _circle.Circle._create_with_graphics(
            graphics=self,
            x=x,
            y=y,
            radius=radius,
            variable_name_suffix=variable_name_suffix,
        )
        return circle

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=3, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=3, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=4, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=4, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=5, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_ellipse(
        self,
        *,
        x: Union[float, Number],
        y: Union[float, Number],
        width: Union[int, Int],
        height: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> "_ellipse.Ellipse":
        """
        Draw an ellipse vector graphic.

        Parameters
        ----------
        x : float or Number
            X-coordinate of the ellipse center.
        y : float or Number
            Y-coordinate of the ellipse center.
        width : Int or int
            Ellipse width.
        height : Int or int
            Ellipse height.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        ellipse : Ellipse
            Created ellipse graphics instance.

        References
        ----------
        - Graphics draw_ellipse interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_ellipse.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> ellipse: ap.Ellipse = sprite.graphics.draw_ellipse(
        ...     x=100, y=100, width=100, height=50
        ... )
        >>> ellipse.x
        Number(100.0)

        >>> ellipse.y
        Number(100.0)

        >>> ellipse.width
        Int(100)

        >>> ellipse.height
        Int(50)

        >>> ellipse.fill_color
        Color("#00aaff")
        """
        ellipse: _ellipse.Ellipse = _ellipse.Ellipse._create_with_graphics(
            graphics=self,
            x=x,
            y=y,
            width=width,
            height=height,
            variable_name_suffix=variable_name_suffix,
        )
        return ellipse

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=3, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def line_to(
        self,
        *,
        x: Union[float, Number],
        y: Union[float, Number],
        variable_name_suffix: str = "",
    ) -> "_polyline.Polyline":
        """
        Draw a line from previous point to specified point (initial
        point is x = 0, y = 0).

        Parameters
        ----------
        x : float or Number
            X destination point to draw a line.
        y : float or Number
            Y destination point to draw a line.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        line : Polyline
            Line graphics instance.

        References
        ----------
        - Graphics move_to and line_to interfaces
            - https://simon-ritchie.github.io/apysc/en/graphics_move_to_and_line_to.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)
        >>> line_1: ap.Polyline = sprite.graphics.move_to(x=50, y=50)
        >>> line_2: ap.Polyline = sprite.graphics.line_to(x=150, y=50)
        >>> line_3: ap.Polyline = sprite.graphics.line_to(x=50, y=150)
        >>> line_1 == line_2 == line_3
        True

        >>> line_1.line_color
        Color("#ffffff")

        >>> line_1.line_thickness
        Int(5)
        """
        if self._current_line is None:
            first_point: Point2D = Point2D(
                x=0, y=0, variable_name_suffix=variable_name_suffix
            )
            second_point: Point2D = Point2D(
                x=x, y=y, variable_name_suffix=variable_name_suffix
            )
            self._current_line = _polyline.Polyline._create_with_graphics(
                graphics=self,
                points=[first_point, second_point],
                variable_name_suffix=variable_name_suffix,
            )
        else:
            self._current_line.append_line_point(x=x, y=y)
            if variable_name_suffix != "":
                self._current_line._variable_name_suffix = variable_name_suffix
        return self._current_line

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=3, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def move_to(
        self,
        *,
        x: Union[float, Number],
        y: Union[float, Number],
        variable_name_suffix: str = "",
    ) -> "_polyline.Polyline":
        """
        Move a line position to a specified point.

        Parameters
        ----------
        x : float or Number
            X destination point to move.
        y : float or Number
            Y destination point to move.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        line : Polyline
            Line graphics instance.

        References
        ----------
        - Graphics move_to and line_to interfaces
            - https://simon-ritchie.github.io/apysc/en/graphics_move_to_and_line_to.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)
        >>> line_1: ap.Polyline = sprite.graphics.move_to(x=50, y=50)
        >>> line_2: ap.Polyline = sprite.graphics.line_to(x=150, y=50)
        >>> line_1 == line_2
        True

        >>> line_1.line_color
        Color("#ffffff")

        >>> line_1.line_thickness
        Int(5)
        """
        point: Point2D = Point2D(x=x, y=y, variable_name_suffix=variable_name_suffix)
        polyline: _polyline.Polyline = _polyline.Polyline._create_with_graphics(
            graphics=self, points=[point], variable_name_suffix=variable_name_suffix
        )
        self._current_line = polyline
        return polyline

    @final
    def _reset_each_line_settings(self) -> None:
        """
        Reset each line settings (e.g., LineDotSetting, LineDashSetting,
        and so on).

        Notes
        -----
        This interface does not append an expression.
        """
        self._line_dot_setting = None
        self._line_dash_setting = None
        self._line_round_dot_setting = None
        self._line_dash_dot_setting = None

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=5, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_line(
        self,
        *,
        x_start: Union[float, Number],
        y_start: Union[float, Number],
        x_end: Union[float, Number],
        y_end: Union[float, Number],
        variable_name_suffix: str = "",
    ) -> "_line.Line":
        """
        Draw a normal line vector graphic.

        Notes
        -----
        - This interface ignores line settings, like
            the `LineDotSetting`, `LineDashSetting`.

        Parameters
        ----------
        x_start : float or Number
            Line start x-coordinate.
        y_start : float or Number
            Line start y-coordinate.
        x_end : float or Number
            Line end x-coordinate.
        y_end : float or Number
            Line end y-coordinate.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        line : Line
            Created line graphics instance.

        References
        ----------
        - Graphics draw_line interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_line.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50
        ... )
        >>> line.line_color
        Color("#ffffff")

        >>> line.line_thickness
        Int(5)
        """
        snapshot_name: str = self._get_next_snapshot_name()
        self._run_all_make_snapshot_methods(snapshot_name=snapshot_name)
        self._reset_each_line_settings()
        line: _line.Line = _line.Line._create_with_graphics(
            graphics=self,
            start_point=Point2D(
                x=x_start, y=y_start, variable_name_suffix=variable_name_suffix
            ),
            end_point=Point2D(
                x=x_end, y=y_end, variable_name_suffix=variable_name_suffix
            ),
            variable_name_suffix=variable_name_suffix,
        )
        self._run_all_revert_methods(snapshot_name=snapshot_name)
        return line

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=5, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=5, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=6, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_dotted_line(
        self,
        *,
        x_start: Union[float, Number],
        y_start: Union[float, Number],
        x_end: Union[float, Number],
        y_end: Union[float, Number],
        dot_size: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> "_line.Line":
        """
        Draw a dotted line vector graphics.

        Notes
        -----
        - This interface ignores line settings, like the
            `LineDashSetting`, except `LineDotSetting`.

        Parameters
        ----------
        x_start : float or Number
            Line start x-coordinate.
        y_start : float or Number
            Line start y-coordinate.
        x_end : float or Number
            Line end x-coordinate.
        y_end : float or Number
            Line end y-coordinate.
        dot_size : Int or int
            Dot size.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        line : Line
            Created line graphics instance.

        References
        ----------
        - Graphics draw_dotted_line interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_dotted_line.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)
        >>> line: ap.Line = sprite.graphics.draw_dotted_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50, dot_size=5
        ... )
        >>> line.line_color
        Color("#ffffff")

        >>> line.line_thickness
        Int(5)

        >>> line.line_dot_setting.dot_size
        Int(5)
        """
        from apysc._display.line_dot_setting import LineDotSetting

        snapshot_name: str = self._get_next_snapshot_name()
        self._run_all_make_snapshot_methods(snapshot_name=snapshot_name)
        self._reset_each_line_settings()
        self._line_dot_setting = LineDotSetting(
            dot_size=dot_size, variable_name_suffix=variable_name_suffix
        )
        line: _line.Line = _line.Line._create_with_graphics(
            graphics=self,
            start_point=Point2D(
                x=x_start, y=y_start, variable_name_suffix=variable_name_suffix
            ),
            end_point=Point2D(
                x=x_end, y=y_end, variable_name_suffix=variable_name_suffix
            ),
            variable_name_suffix=variable_name_suffix,
        )
        self._run_all_revert_methods(snapshot_name=snapshot_name)
        return line

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=5, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=5, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=6, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=6, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=7, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_dashed_line(
        self,
        *,
        x_start: Union[float, Number],
        y_start: Union[float, Number],
        x_end: Union[float, Number],
        y_end: Union[float, Number],
        dash_size: Union[int, Int],
        space_size: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> "_line.Line":
        """
        Draw a dashed line vector graphics.

        Notes
        -----
        - This interface ignores line settings, like
        the `LineDotSetting`, except `LineDashSetting`.

        Parameters
        ----------
        x_start : float or Number
            Line start x-coordinate.
        y_start : float or Number
            Line start y-coordinate.
        x_end : float or Number
            Line end x-coordinate.
        y_end : float or Number
            Line end y-coordinate.
        dash_size : Int or int
            Dash size.
        space_size : Int or int
            Blank space size between dashes.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        line : Line
            Created line graphics instance.

        References
        ----------
        - Graphics draw_dashed_line interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_dashed_line.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)
        >>> line: ap.Line = sprite.graphics.draw_dashed_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50, dash_size=5, space_size=2
        ... )
        >>> line.line_color
        Color("#ffffff")

        >>> line.line_dash_setting.dash_size
        Int(5)

        >>> line.line_dash_setting.space_size
        Int(2)
        """
        from apysc._display.line_dash_setting import LineDashSetting

        snapshot_name: str = self._get_next_snapshot_name()
        self._run_all_make_snapshot_methods(snapshot_name=snapshot_name)
        self._reset_each_line_settings()
        self._line_dash_setting = LineDashSetting(
            dash_size=dash_size,
            space_size=space_size,
            variable_name_suffix=variable_name_suffix,
        )
        line: _line.Line = _line.Line._create_with_graphics(
            graphics=self,
            start_point=Point2D(
                x=x_start, y=y_start, variable_name_suffix=variable_name_suffix
            ),
            end_point=Point2D(
                x=x_end, y=y_end, variable_name_suffix=variable_name_suffix
            ),
            variable_name_suffix=variable_name_suffix,
        )
        self._run_all_revert_methods(snapshot_name=snapshot_name)
        return line

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=5, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=5, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=6, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=6, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=7, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_round_dotted_line(
        self,
        *,
        x_start: Union[float, Number],
        y_start: Union[float, Number],
        x_end: Union[float, Number],
        y_end: Union[float, Number],
        round_size: Union[int, Int],
        space_size: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> "_line.Line":
        """
        Draw a round-dotted line vector graphics.

        Notes
        -----
        This interface ignores line settings, like the
        `LineDotSetting`, except `LineRoundDotSetting`.

        Parameters
        ----------
        x_start : float or Number
            Line start x-coordinate.
        y_start : float or Number
            Line start y-coordinate.
        x_end : float or Number
            Line end x-coordinate.
        y_end : float or Number
            Line end y-coordinate.
        round_size : Int or int
            Dot round size.
        space_size : Int or int
            Blank space size between dots.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        line : Line
            Created line graphics instance.

        References
        ----------
        - Graphics draw_round_dotted_line interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_round_dotted_line.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)
        >>> line: ap.Line = sprite.graphics.draw_round_dotted_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50, round_size=6, space_size=3
        ... )
        >>> line.line_color
        Color("#ffffff")

        >>> line.line_round_dot_setting.round_size
        Int(6)

        >>> line.line_round_dot_setting.space_size
        Int(3)
        """
        from apysc._display.line_round_dot_setting import LineRoundDotSetting

        snapshot_name: str = self._get_next_snapshot_name()
        self._run_all_make_snapshot_methods(snapshot_name=snapshot_name)
        self._reset_each_line_settings()
        self._line_round_dot_setting = LineRoundDotSetting(
            round_size=round_size,
            space_size=space_size,
            variable_name_suffix=variable_name_suffix,
        )
        line: _line.Line = _line.Line._create_with_graphics(
            graphics=self,
            start_point=Point2D(
                x=x_start, y=y_start, variable_name_suffix=variable_name_suffix
            ),
            end_point=Point2D(
                x=x_end, y=y_end, variable_name_suffix=variable_name_suffix
            ),
            variable_name_suffix=variable_name_suffix,
        )
        self._run_all_revert_methods(snapshot_name=snapshot_name)
        return line

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=5, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=5, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=6, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=6, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=7, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=7, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=8, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_dash_dotted_line(
        self,
        *,
        x_start: Union[float, Number],
        y_start: Union[float, Number],
        x_end: Union[float, Number],
        y_end: Union[float, Number],
        dot_size: Union[int, Int],
        dash_size: Union[int, Int],
        space_size: Union[int, Int],
        variable_name_suffix: str = "",
    ) -> "_line.Line":
        """
        Draw a dash-dotted (1-dot chain) line vector graphics.

        Parameters
        ----------
        x_start : float or Number
            Line start x-coordinate.
        y_start : float or Number
            Line start y-coordinate.
        x_end : float or Number
            Line end x-coordinate.
        y_end : float or Number
            Line end y-coordinate.
        dot_size : Int or int
            Dot size.
        dash_size : Int or int
            Dash size.
        space_size : Int or int
            Blank space size between dots and dashes.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        line : Line
            Created line graphics instance.

        References
        ----------
        - Graphics draw_dash_dotted_line interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_dash_dotted_line.html # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)
        >>> line: ap.Line = sprite.graphics.draw_dash_dotted_line(
        ...     x_start=50,
        ...     y_start=50,
        ...     x_end=150,
        ...     y_end=50,
        ...     dot_size=2,
        ...     dash_size=5,
        ...     space_size=3,
        ... )
        >>> line.line_color
        Color("#ffffff")

        >>> line.line_dash_dot_setting.dot_size
        Int(2)

        >>> line.line_dash_dot_setting.dash_size
        Int(5)

        >>> line.line_dash_dot_setting.space_size
        Int(3)
        """
        from apysc._display.line_dash_dot_setting import LineDashDotSetting

        snapshot_name: str = self._get_next_snapshot_name()
        self._run_all_make_snapshot_methods(snapshot_name=snapshot_name)
        self._reset_each_line_settings()
        self._line_dash_dot_setting = LineDashDotSetting(
            dot_size=dot_size,
            dash_size=dash_size,
            space_size=space_size,
            variable_name_suffix=variable_name_suffix,
        )
        line: _line.Line = _line.Line._create_with_graphics(
            graphics=self,
            start_point=Point2D(
                x=x_start, y=y_start, variable_name_suffix=variable_name_suffix
            ),
            end_point=Point2D(
                x=x_end, y=y_end, variable_name_suffix=variable_name_suffix
            ),
            variable_name_suffix=variable_name_suffix,
        )
        self._run_all_revert_methods(snapshot_name=snapshot_name)
        return line

    @final
    @arg_validation_decos.are_point_2ds(arg_position_index=1)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_polygon(
        self,
        *,
        points: Union[List[Point2D], Array[Point2D]],
        variable_name_suffix: str = "",
    ) -> "_polyg.Polygon":
        """
        Draw a polygon vector graphic. This interface is similar
        to the Polyline class (created by `move_to` or `line_to`).
        But unlike that, this interface connects the last point
        and the start point.

        Parameters
        ----------
        points : list of Point2D or Array.
            Polygon vertex points.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        polygon : Polygon
            Created polygon graphics instance.

        References
        ----------
        - Graphics draw_polygon interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_polygon.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> polygon: ap.Polygon = sprite.graphics.draw_polygon(
        ...     points=[
        ...         ap.Point2D(x=25, y=0),
        ...         ap.Point2D(x=0, y=50),
        ...         ap.Point2D(x=50, y=50),
        ...     ]
        ... )
        >>> polygon.fill_color
        Color("#00aaff")
        """
        polygon: _polyg.Polygon = _polyg.Polygon._create_with_graphics(
            graphics=self, points=points, variable_name_suffix=variable_name_suffix
        )
        return polygon

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    @arg_validation_decos.is_num(arg_position_index=4, optional=False)
    @arg_validation_decos.is_num(arg_position_index=5, optional=False)
    @arg_validation_decos.is_num(arg_position_index=6, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=7, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_triangle(
        self,
        *,
        x1: Union[float, Number],
        y1: Union[float, Number],
        x2: Union[float, Number],
        y2: Union[float, Number],
        x3: Union[float, Number],
        y3: Union[float, Number],
        variable_name_suffix: str = "",
    ) -> _triangle.Triangle:
        """
        Draw a triangle vector graphic.

        Parameters
        ----------
        x1 : Union[float, Number]
            First vertex's x coordinate.
        y1 : Union[float, Number]
            First vertex's y coordinate.
        x2 : Union[float, Number]
            Second vertex's x coordinate.
        y2 : Union[float, Number]
            Second vertex's y coordinate.
        x3 : Union[float, Number]
            Third vertex's x coordinate.
        y3 : Union[float, Number]
            Third vertex's y coordinate.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        triangle : Triangle
            Created triangle graphics instance.

        References
        ----------
        - Triangle class
            - https://simon-ritchie.github.io/apysc/en/triangle.html
        - Graphics draw_triangle interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_triangle.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.7)
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5, alpha=0.5)
        >>> triangle: ap.Triangle = sprite.graphics.draw_triangle(
        ...     x1=75,
        ...     y1=50,
        ...     x2=25,
        ...     y2=100,
        ...     x3=100,
        ...     y3=100,
        ... )
        >>> triangle.x1
        Number(75.0)
        >>> triangle.y1 = ap.Number(30)
        >>> triangle.y1
        Number(30.0)
        >>> triangle.fill_color
        Color("#00aaff")
        """
        triangle: _triangle.Triangle = _triangle.Triangle._create_with_graphics(
            graphics=self,
            x1=x1,
            y1=y1,
            x2=x2,
            y2=y2,
            x3=x3,
            y3=y3,
            variable_name_suffix=variable_name_suffix,
        )
        return triangle

    @final
    @arg_validation_decos.is_valid_path_data_list(arg_position_index=1)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def draw_path(
        self, *, path_data_list: List[PathDataBase], variable_name_suffix: str = ""
    ) -> "_path.Path":
        """
        Draw a path vector graphics.

        Parameters
        ----------
        path_data_list : list of PathDataBase
            Target path data settings, such as the ap.PathData.MoveTo.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        path : Path
            Created path graphics instance.

        References
        ----------
        - Path class
            - https://simon-ritchie.github.io/apysc/en/path.html
        - Graphics draw_path interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_path.html
        - PathMoveTo class
            - https://simon-ritchie.github.io/apysc/en/path_move_to.html
        - PathLineTo class
            - https://simon-ritchie.github.io/apysc/en/path_line_to.html
        - PathHorizontal class
            - https://simon-ritchie.github.io/apysc/en/path_horizontal.html
        - PathVertical class
            - https://simon-ritchie.github.io/apysc/en/path_vertical.html
        - PathClose class
            - https://simon-ritchie.github.io/apysc/en/path_close.html
        - PathBezier2D class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_2d.html
        - PathBezier2DContinual class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_2d_continual.html
        - PathBezier3D class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_3d.html
        - PathBezier3DContinual class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_3d_continual.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=3)
        >>> path: ap.Path = sprite.graphics.draw_path(
        ...     path_data_list=[
        ...         ap.PathMoveTo(x=0, y=50),
        ...         ap.PathBezier2D(control_x=50, control_y=0, dest_x=100, dest_y=50),
        ...     ]
        ... )
        """
        path: _path.Path = _path.Path._create_with_graphics(
            graphics=self,
            path_data_list=path_data_list,
            variable_name_suffix=variable_name_suffix,
        )
        return path

    @final
    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Graphics("<variable_name>")`).
        """
        repr_str: str = f'{Graphics.__name__}("{self.variable_name}")'
        return repr_str

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "Graphics":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        graphics : Graphics
            An initialized graphics instance.
        """
        from apysc._type.boolean import Boolean

        sprite_: sprite.Sprite = sprite.Sprite()
        graphics: Graphics = Graphics(parent=sprite_)
        graphics.visible = Boolean(False)
        return graphics



"""The class implementation for the outlined `pets` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPetsOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `pets` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><circle cx="4.5" cy="9.5" r="2.5"/><circle cx="9" cy="5.5" r="2.5"/><circle cx="15" cy="5.5" r="2.5"/><circle cx="19.5" cy="9.5" r="2.5"/><path d="M17.34 14.86c-.87-1.02-1.6-1.89-2.48-2.91-.46-.54-1.05-1.08-1.75-1.32-.11-.04-.22-.07-.33-.09-.25-.04-.52-.04-.78-.04s-.53 0-.79.05c-.11.02-.22.05-.33.09-.7.24-1.28.78-1.75 1.32-.87 1.02-1.6 1.89-2.48 2.91-1.31 1.31-2.92 2.76-2.62 4.79.29 1.02 1.02 2.03 2.33 2.32.73.15 3.06-.44 5.54-.44h.18c2.48 0 4.81.58 5.54.44 1.31-.29 2.04-1.31 2.33-2.32.31-2.04-1.3-3.49-2.61-4.8z"/></svg>'  # noqa



"""The class implementation for the `alarm_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAlarmOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `alarm_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M12 6c3.87 0 7 3.13 7 7 0 .84-.16 1.65-.43 2.4l1.52 1.52c.58-1.19.91-2.51.91-3.92 0-4.97-4.03-9-9-9-1.41 0-2.73.33-3.92.91L9.6 6.43C10.35 6.16 11.16 6 12 6zm10-.28l-4.6-3.86-1.29 1.53 4.6 3.86L22 5.72zM2.92 2.29L1.65 3.57 2.98 4.9l-1.11.93 1.42 1.42 1.11-.94.8.8C3.83 8.69 3 10.75 3 13c0 4.97 4.02 9 9 9 2.25 0 4.31-.83 5.89-2.2l2.2 2.2 1.27-1.27L3.89 3.27l-.97-.98zm13.55 16.1C15.26 19.39 13.7 20 12 20c-3.87 0-7-3.13-7-7 0-1.7.61-3.26 1.61-4.47l9.86 9.86zM8.02 3.28L6.6 1.86l-.86.71 1.42 1.42.86-.71z"/></svg>'  # noqa



"""The class implementation for the `shopping_cart` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialShoppingCartIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `shopping_cart` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M7 18c-1.1 0-1.99.9-1.99 2S5.9 22 7 22s2-.9 2-2-.9-2-2-2zM1 2v2h2l3.6 7.59-1.35 2.45c-.16.28-.25.61-.25.96 0 1.1.9 2 2 2h12v-2H7.42c-.14 0-.25-.11-.25-.25l.03-.12.9-1.63h7.45c.75 0 1.41-.41 1.75-1.03l3.58-6.49c.08-.14.12-.31.12-.48 0-.55-.45-1-1-1H5.21l-.94-2H1zm16 16c-1.1 0-1.99.9-1.99 2s.89 2 1.99 2 2-.9 2-2-.9-2-2-2z"/></svg>'  # noqa



"""The class implementation for the outlined `art_track` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialArtTrackOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `art_track` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M22 13h-8v-2h8v2zm0-6h-8v2h8V7zm-8 10h8v-2h-8v2zm-2-8v6c0 1.1-.9 2-2 2H4c-1.1 0-2-.9-2-2V9c0-1.1.9-2 2-2h6c1.1 0 2 .9 2 2zm-1.5 6l-2.25-3-1.75 2.26-1.25-1.51L3.5 15h7z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: array_clear.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class clear interface": "# Array クラスの clear インターフェイス",
    ##################################################
    "This page explains the `Array` class `clear` method interface.": "このページでは`Array`クラスの`clear`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `clear` method clears an Array's value. This interface makes an Array's value empty.": "`clear`メソッドは配列の値を空にします。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `clear` method requires no arguments.": "`clear`メソッドは呼び出しに特に引数を必要としません。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array = ap.Array([10, 20, 30])\narr.clear()\nassert arr == []\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\narr: ap.Array = ap.Array([10, 20, 30])\narr.clear()\nassert arr == []\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_flip_interfaces.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase flip_x and flip_y interfaces": "# GraphicsBase クラスの flip_x と flip_y インターフェイス",  # noqa
    ##################################################
    "This page explains the `GraphicsBase` class (base class of each graphic, such as the `Rectangle`) `flip_x` and `flip_y` property interfaces.": "このページでは`GraphicsBase`クラス（`Rectangle`などの各グラフィッククラスの基底クラス）の`flip_x`と`flip_y`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `flip_x` property flips an object in the x-axis direction, and the `flip_y` property flips in the y-axis direction.": "`flip_x`属性はオブジェクトを横方向に反転し、`flip_y`属性はオブジェクトを縦方向に反転します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "You can set a `Boolean` value to the `flip_x` and `flip_y` properties.": "`flip_x`と`flip_y`属性には`Boolean`の値を設定することができます。",  # noqa
    ##################################################
    "If you set the `True`, an object becomes flipped. Conversely, if you set the `False`, an object resets flipping.": "もし`True`を指定すれば反転した状態になり、`False`を設定すれ反転がリセットされます。",  # noqa
    ##################################################
    "The getter interface returns a `Boolean` value of a current flipping value.": "getterのインターフェイスでは現在の反転設定の`Boolean`型の値を返却します。",  # noqa
    ##################################################
    "The following example flips the triangle polygon in the x-axis direction and resets per second:": "以下のコード例では1秒ごとに三角形の反転とリセットを行っています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _PolygonOptions(TypedDict):\n    polygon: ap.Polygon\n\n\ndef on_timer(e: ap.TimerEvent, options: _PolygonOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    polygon: ap.Polygon = options["polygon"]\n    flip_x: ap.Boolean = polygon.flip_x\n    flip_x = flip_x.not_\n    polygon.flip_x = flip_x\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\npolygon: ap.Polygon = sprite.graphics.draw_polygon(\n    points=[\n        ap.Point2D(x=50, y=50),\n        ap.Point2D(x=50, y=100),\n        ap.Point2D(x=100, y=75),\n    ]\n)\noptions: _PolygonOptions = {"polygon": polygon}\ntimer: ap.Timer = ap.Timer(on_timer, delay=1000, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="graphics_base_flip_x_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _PolygonOptions(TypedDict):\n    polygon: ap.Polygon\n\n\ndef on_timer(e: ap.TimerEvent, options: _PolygonOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    polygon: ap.Polygon = options["polygon"]\n    flip_x: ap.Boolean = polygon.flip_x\n    flip_x = flip_x.not_\n    polygon.flip_x = flip_x\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\npolygon: ap.Polygon = sprite.graphics.draw_polygon(\n    points=[\n        ap.Point2D(x=50, y=50),\n        ap.Point2D(x=50, y=100),\n        ap.Point2D(x=100, y=75),\n    ]\n)\noptions: _PolygonOptions = {"polygon": polygon}\ntimer: ap.Timer = ap.Timer(on_timer, delay=1000, options=options)\ntimer.start()\n\nap.save_overall_html(dest_dir_path="graphics_base_flip_x_basic_usage/")\n```',  # noqa
    ##################################################
    "The `flip_y` interface behaves the same as the `flip_x` interface, except the axis direction.": "`flip_y`インターフェイスは軸の方向の違いを除いて`flip_x`の員スターフェイスと同様に動作します。",  # noqa
    ##################################################
    "## flip_x property API": "## flip_x 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a boolean value whether the x-axis is flipping or not.<hr>": "横軸に対して反転しているかどうかの真偽値を取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `flip_x`: Boolean": "- `flip_x`: Boolean",
    ##################################################
    "  - A boolean value, whether the x-axis is flipping or not.": "  - 横方向に反転しているかどうかのBooleanの真偽値。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=0, y=0),\n...         ap.Point2D(x=0, y=50),\n...         ap.Point2D(x=50, y=25),\n...     ]\n... )\n>>> polygon.flip_x = ap.Boolean(True)\n>>> polygon.flip_x\nBoolean(True)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=0, y=0),\n...         ap.Point2D(x=0, y=50),\n...         ap.Point2D(x=50, y=25),\n...     ]\n... )\n>>> polygon.flip_x = ap.Boolean(True)\n>>> polygon.flip_x\nBoolean(True)\n```',  # noqa
    ##################################################
    "## flip_y property API": "## flip_y 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a boolean value whether the y-axis is flipping or not.<hr>": "縦軸に対して反転しているかどうかの真偽値を取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `flip_y`: Boolean": "- `flip_y`: Boolean",
    ##################################################
    "  - A boolean value whether the y-axis is flipping or not.": "  - 縦方向に反転しているかどうかのBooleanの真偽値。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=0, y=0),\n...         ap.Point2D(x=50, y=0),\n...         ap.Point2D(x=25, y=50),\n...     ]\n... )\n>>> polygon.flip_y = ap.Boolean(True)\n>>> polygon.flip_y\nBoolean(True)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=0, y=0),\n...         ap.Point2D(x=50, y=0),\n...         ap.Point2D(x=25, y=50),\n...     ]\n... )\n>>> polygon.flip_y = ap.Boolean(True)\n>>> polygon.flip_y\nBoolean(True)\n```',  # noqa
}



"""The class implementation for the `contact_support` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialContactSupportIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `contact_support` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M11.5 2C6.81 2 3 5.81 3 10.5S6.81 19 11.5 19h.5v3c4.86-2.34 8-7 8-11.5C20 5.81 16.19 2 11.5 2zm1 14.5h-2v-2h2v2zm0-3.5h-2c0-3.25 3-3 3-5 0-1.1-.9-2-2-2s-2 .9-2 2h-2c0-2.21 1.79-4 4-4s4 1.79 4 4c0 2.5-3 2.75-3 5z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: color.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Color class": "# Color クラス",
    ##################################################
    "This page explains the `Color` class.": "このページでは`Color`クラスについて説明します。",
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `Color` class handles color settings.": "`Color`クラスの色の設定を扱います。",
    ##################################################
    "Color settings, such as the `fill_color` or `line_color` arguments or properties, require its value.": "色の設定、例えば`fill_color`や`line_color`などの引数や属性はこの値を必要とします。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The constructor of the `Color` class requires a hexadecimal color code string, for example, `#00aaff`.": "`Color`クラスのコンストラクタでは例えば`#00aaff`などの16進数のカラーコードを必要とします。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nleft_rectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#00aaff"),\n)\n\nright_rectangle: ap.Rectangle = ap.Rectangle(\n    x=150,\n    y=50,\n    width=50,\n    height=50,\n    line_color=ap.Color("#ffffff"),\n    line_thickness=3,\n)\n\nap.save_overall_html(dest_dir_path="./color_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nleft_rectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#00aaff"),\n)\n\nright_rectangle: ap.Rectangle = ap.Rectangle(\n    x=150,\n    y=50,\n    width=50,\n    height=50,\n    line_color=ap.Color("#ffffff"),\n    line_thickness=3,\n)\n\nap.save_overall_html(dest_dir_path="./color_basic_usage/")\n```',  # noqa
    ##################################################
    "## Acceptable hexadecimal color codes": "## 受け付けられる16進数のカラーコード",
    ##################################################
    "Color code is acceptable like the following list:": "以下のようなカラーコードが受け付けられます:",
    ##################################################
    "- Six characters, e.g., `#00aaff`.": "- `#00aaff`などの6文字による指定。",
    ##################################################
    "- Three characters, e.g., `#0af` (this becomes `#00aaff`).": "- `#0af`などの3文字による指定（これは`#00aaff`と同じ値として扱われます）。",  # noqa
    ##################################################
    "- A single character, e.g., `#5` (this becomes `#000005`).": "- `#5`などの1文字による指定（これは`000005`と同じ値として扱われます）。",  # noqa
    ##################################################
    "- A skipped `#` symbol string, e.g., `0af` (this becomes `#00aaff`).": "- `0af`などの`#`記号を省略した指定（これは`#00aaff`と同じ値として扱われます）。",  # noqa
    ##################################################
    "- The `COLORLESS` constant (this setting clears a color setting).": "- `COLORLESS`定数（この設定は色の設定を削除します）。",  # noqa
    ##################################################
    "## Color constructor API": "## Color クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "The color class implementation.<hr>": "色のクラスの実装です。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: str or String": "- `value`: str or String",
    ##################################################
    "  - A hexadecimal color code string (e.g., '#000000').": "  - 16進数の色の文字列（例 : '#000000'）。",  # noqa
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> color: ap.Color = ap.Color("#0af")\n>>> color\nColor("#00aaff")\n\n>>> color = ap.Color("#ffffff")\n>>> color\nColor("#ffffff")\n```': '```py\n>>> import apysc as ap\n>>> color: ap.Color = ap.Color("#0af")\n>>> color\nColor("#00aaff")\n\n>>> color = ap.Color("#ffffff")\n>>> color\nColor("#ffffff")\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Colors class](https://simon-ritchie.github.io/apysc/en/colors.html)": "- [Colors クラス](https://simon-ritchie.github.io/apysc/jp/jp_colors.html)",  # noqa
    ##################################################
    "- [MaterialDesignColors class](https://simon-ritchie.github.io/apysc/en/material_design_colors.html)": "- [MaterialDesignColors クラス](https://simon-ritchie.github.io/apysc/jp/jp_material_design_colors.html)",  # noqa
    ##################################################
    "- [COLORLESS constant](https://simon-ritchie.github.io/apysc/en/colorless.html)": "- [COLORLESS 定数](https://simon-ritchie.github.io/apysc/jp/jp_colorless.html)",  # noqa
    ##################################################
    "- [Color class from_rgb class method](https://simon-ritchie.github.io/apysc/en/color_from_rgb.html)": "- [Color クラスの from_rgb クラスメソッド](https://simon-ritchie.github.io/apysc/jp/jp_color_from_rgb.html)",  # noqa
    ##################################################
    "- [Color class red_color property](https://simon-ritchie.github.io/apysc/en/red_color.html)": "- [Color クラスの red_color 属性](https://simon-ritchie.github.io/apysc/jp/jp_red_color.html)",  # noqa
    ##################################################
    "- [Color class green_color property](https://simon-ritchie.github.io/apysc/en/green_color.html)": "- [Color クラスの green_color 属性](https://simon-ritchie.github.io/apysc/jp/jp_green_color.html)",  # noqa
    ##################################################
    "- [Color class blue_color property](https://simon-ritchie.github.io/apysc/en/blue_color.html)": "- [Color クラスの blue_color 属性](https://simon-ritchie.github.io/apysc/jp/jp_blue_color.html)",  # noqa
}



"""The mix-in class implementation for the `SvgText`'s
`_skip_line_thickness_expression_appending` attribute.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int


class SvgTextSkipLineThicknessExpAppendingMixIn:
    _skip_line_thickness_expression_appending: bool = False

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_line_thickness_expression_skipping_attr(
        self,
        *,
        line_thickness: Optional[Union[int, Int]],
    ) -> None:
        """
        Set a boolean attribute, whether to skip a line-thickness expression's
        appending or not.

        Parameters
        ----------
        line_thickness : Optional[Union[int, Int]]
            A line-thickness setting.
        """
        if line_thickness is None:
            self._skip_line_thickness_expression_appending = True
            return
        self._skip_line_thickness_expression_appending = False



# mypy: disable-error-code=type-var

"""Common testing helper implementations.
"""

import functools
import time
from random import randint
from typing import Any
from typing import Callable
from typing import Dict
from typing import Optional
from typing import Type
from typing import TypeVar
from typing import cast

import pytest


def make_blank_file(*, file_path: str) -> None:
    """
    Make a blank file. If there is no directory, also create
    a parent directory.

    Parameters
    ----------
    file_path : str
        File path to make.
    """
    from apysc._file import file_util

    file_util.save_plain_txt(txt="", file_path=file_path)


def _assert_has_attr(*, any_obj: Any, attr_name: str) -> None:
    """
    Check object has a specified attribute.

    Parameters
    ----------
    any_obj : *
        Any object to check.
    attr_name : str
        Expected attribute name.
    """
    msg: str = "Expected attribute does not exists." f"\nAttribute name: {attr_name}"
    assert hasattr(any_obj, attr_name), msg


def assert_attrs(*, expected_attrs: Dict[str, Any], any_obj: Any) -> None:
    """
    Check a specified object's attributes.

    Parameters
    ----------
    expected_attrs : dict
        A dict that has attribute names in key and expected values in value.
    any_obj : *
        Any object to check.

    Raises
    ------
    AssertionError
        If an expected attribute value does not exist or differ.
    """
    for attr_name, expected_value in expected_attrs.items():
        _assert_has_attr(any_obj=any_obj, attr_name=attr_name)

        attr_val: Any = getattr(any_obj, attr_name)
        msg: str = (
            "Attribute value is different from expected value."
            f"\nAttribute name: {attr_name}"
            f"\nAttribute value: {attr_val}"
            f"\nExpected value: {expected_value}"
        )
        assert attr_val == expected_value, msg


def assert_attrs_type(*, expected_types: Dict[str, Type], any_obj: Any) -> None:
    """
    Check a specified object's attribute types.

    Parameters
    ----------
    expected_types : dict
        A dict that has attribute names in key and expected types in value.
    any_obj : *
        Any object to check.

    Raises
    ------
    AssertionError
        If any attribute type differs from an expected type.
    """
    for attr_name, expected_type in expected_types.items():
        _assert_has_attr(any_obj=any_obj, attr_name=attr_name)

        attr_val: Any = getattr(any_obj, attr_name)
        msg: str = (
            "Attribute type is different from expected type."
            f"\nAttribute name: {attr_name}"
            f"\nAttribute type: {type(attr_val)}"
            f"\nExpected type: {expected_type}"
        )
        assert isinstance(attr_val, expected_type), msg


def assert_raises(
    *,
    expected_error_class: type,
    callable_: Callable,
    match: Optional[str] = None,
    **kwargs: Any,
) -> None:
    """
    Check that a specified callable raises an exception.

    Parameters
    ----------
    expected_error_class : type
        Expected error class, for instance, ValueError, Exception, etc.
    callable_ : callable
        Target function or method.
    match : str or None, default None
        Error message's regular expression match pattern.
    **kwargs : dict, optional
        Keyword arguments to pass to the function or method.

    Raises
    ------
    AssertionError
        If a specified interface does not raise an error.
    """
    _kwargs: Dict[str, Any] = {}
    if match is not None:
        _kwargs["match"] = match
    with pytest.raises(expected_error_class, **_kwargs):  # type: ignore
        callable_(**kwargs)


# pyright: reportInvalidTypeVarUse=false
_Callable = TypeVar("_Callable", bound=Callable)


def apply_test_settings(
    *,
    retrying_max_attempts_num: int = 15,
    retrying_sleep_seconds: Optional[float] = None,
) -> _Callable:
    """
    Apply each test setting to a test function.
    This function is a decorator function.

    Parameters
    ----------
    retrying_max_attempts_num : int, optional
        A maximum number of retry attempts.
    retrying_sleep_seconds : Optional[float], optional
        A Sleep seconds of retrying (Maximum seconds is 10).

    Returns
    -------
    wrapped : Callable
        Wrapped callable object.

    Raises
    ------
    ValueError
        If a specified `retrying_sleep_seconds` is greater than 10.
    """
    from apysc._display.stage import Stage

    if retrying_sleep_seconds is None:
        retrying_sleep_seconds = randint(10, 3000) / 1000
    _validate_retrying_sleep_seconds(retrying_sleep_seconds=retrying_sleep_seconds)
    retrying_sleep_seconds_: float = cast(float, retrying_sleep_seconds)

    def wrapped(callable_: _Callable) -> _Callable:
        @functools.wraps(callable_)
        def inner_wrapped(*args: Any, **kwargs: Any) -> Any:
            result: Any = None
            for i in range(retrying_max_attempts_num + 1):
                if i < retrying_max_attempts_num:
                    try:
                        Stage()
                        result = callable_(*args, **kwargs)
                        break
                    except Exception:
                        time.sleep(retrying_sleep_seconds_)
                        continue
                Stage()
                result = callable_(*args, **kwargs)
                break
            return result

        return inner_wrapped  # type: ignore

    return wrapped  # type: ignore


def _validate_retrying_sleep_seconds(*, retrying_sleep_seconds: float) -> None:
    """
    Validate whether specified retrying sleep seconds is not greater than 10.

    Parameters
    ----------
    retrying_sleep_seconds : float
        Sleep seconds of retrying.

    Raises
    ------
    ValueError
        If a specified `retrying_sleep_seconds` is greater than 10.
    """
    if retrying_sleep_seconds <= 10:
        return
    raise ValueError(
        "A specified `retrying_sleep_seconds` argument is greater than 10: "
        f"{retrying_sleep_seconds}"
    )



"""The mix-in class implementation for the material button's icon-related
methods.
"""

from typing import Optional
from typing import cast

from apysc._color.color import Color
from apysc._display.child_mixin import ChildMixIn
from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._geom.rectangle_geom import RectangleGeom
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class _InstanceStub(
    XMixIn,
    YMixIn,
    ChildMixIn,
    VariableNameSuffixAttrOrVarMixIn,
):
    pass


class MaterialButtonIconMixIn:
    """
    The mix-in class implementation for the material button's
    icon-related methods.
    """

    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=1, optional=True)
    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=2, optional=True)
    @arg_validation_decos.is_rectangle_geom(arg_position_index=3)
    def _locate_icons(
        self,
        *,
        prefix_icon: Optional[FixedHtmlSvgIconBase],
        suffix_icon: Optional[FixedHtmlSvgIconBase],
        background_bounding_box: RectangleGeom,
    ) -> None:
        """
        Locate each icon.

        Parameters
        ----------
        prefix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the left side of the label.
        suffix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the right side of the label.
        background_bounding_box : RectangleGeom
            A bounding box of the background.
        """
        from apysc._material_design.button.material_button_const import (
            ICON_OUTER_PADDING_WIDTH,
        )
        from apysc._material_design.button.material_button_const import ICON_SIZE
        from apysc._material_design.button.material_button_const import ICON_Y
        from apysc._type.number import Number

        self_instance: _InstanceStub = cast(_InstanceStub, self)
        if prefix_icon is not None:
            suffix: str = self_instance._get_attr_or_variable_name_suffix(
                value_identifier="prefix_icon_x"
            )
            prefix_icon.x = Number(
                ICON_OUTER_PADDING_WIDTH, variable_name_suffix=suffix
            )

            suffix = self_instance._get_attr_or_variable_name_suffix(
                value_identifier="prefix_icon_y"
            )
            prefix_icon.y = Number(ICON_Y, variable_name_suffix=suffix)

        if suffix_icon is not None:
            suffix = self_instance._get_attr_or_variable_name_suffix(
                value_identifier="suffix_icon_x"
            )
            suffix_icon.x = Number(
                background_bounding_box.width - ICON_OUTER_PADDING_WIDTH - ICON_SIZE,
                variable_name_suffix=suffix,
            )

            suffix = self_instance._get_attr_or_variable_name_suffix(
                value_identifier="suffix_icon_y"
            )
            suffix_icon.y = Number(ICON_Y, variable_name_suffix=suffix)

    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=1, optional=True)
    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=2, optional=True)
    def _add_icons(
        self,
        *,
        prefix_icon: Optional[FixedHtmlSvgIconBase],
        suffix_icon: Optional[FixedHtmlSvgIconBase],
    ) -> None:
        """
        Add each icon to this button.

        Parameters
        ----------
        prefix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the left side of the label.
        suffix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the right side of the label.
        """
        self_instance: _InstanceStub = cast(_InstanceStub, self)
        if prefix_icon is not None:
            self_instance.add_child(child=prefix_icon)
        if suffix_icon is not None:
            self_instance.add_child(child=suffix_icon)

    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=1, optional=True)
    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=2, optional=True)
    def _resize_icon_size(
        self,
        *,
        prefix_icon: Optional[FixedHtmlSvgIconBase],
        suffix_icon: Optional[FixedHtmlSvgIconBase],
    ) -> None:
        """
        Resize the icon size if each icon is specified.

        Parameters
        ----------
        prefix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the left side of the label.
        suffix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the right side of the label.
        """
        from apysc._material_design.button.material_button_const import ICON_SIZE
        from apysc._type.int import Int

        if prefix_icon is not None:
            prefix_icon.width = Int(ICON_SIZE, variable_name_suffix="prefix_icon_width")
            prefix_icon.height = Int(
                ICON_SIZE, variable_name_suffix="prefix_icon_height"
            )
        if suffix_icon is not None:
            suffix_icon.width = Int(ICON_SIZE, variable_name_suffix="suffix_icon_width")
            suffix_icon.height = Int(
                ICON_SIZE, variable_name_suffix="suffix_icon_height"
            )

    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=2, optional=True)
    @arg_validation_decos.is_fixed_html_svg_icon(arg_position_index=3, optional=True)
    def _set_fill_color_to_icons(
        self,
        *,
        color: Color,
        prefix_icon: Optional[FixedHtmlSvgIconBase],
        suffix_icon: Optional[FixedHtmlSvgIconBase],
    ) -> None:
        """
        Set a fill-color to the icons.

        Parameters
        ----------
        color : Color
            A color to set.
        prefix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the left side of the label.
        suffix_icon : Optional[FixedHtmlSvgIconBase]
            An icon to display on the right side of the label.
        """
        if prefix_icon is not None:
            prefix_icon.fill_color = color
        if suffix_icon is not None:
            suffix_icon.fill_color = color



"""The class implementation for the `location_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLocationOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `location_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0zm11.75 11.47l-.11-.11z" fill="none"/><path d="M12 6.5c1.38 0 2.5 1.12 2.5 2.5 0 .74-.33 1.39-.83 1.85l3.63 3.63c.98-1.86 1.7-3.8 1.7-5.48 0-3.87-3.13-7-7-7-1.98 0-3.76.83-5.04 2.15l3.19 3.19c.46-.52 1.11-.84 1.85-.84zm4.37 9.6l-4.63-4.63-.11-.11L3.27 3 2 4.27l3.18 3.18C5.07 7.95 5 8.47 5 9c0 5.25 7 13 7 13s1.67-1.85 3.38-4.35L18.73 21 20 19.73l-3.63-3.63z"/></svg>'  # noqa



"""Each interface to get an apysc value from a Python built-in one.
"""

from typing import Union

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String


@add_debug_info_setting(module_name=__name__)
def get_copied_int_from_builtin_val(
    *, integer: Union[int, Int], variable_name_suffix: str = ""
) -> Int:
    """
    Get a copied Int value from a Python built-in int.

    Parameters
    ----------
    integer : int or Int
        Target integer value.
    variable_name_suffix : str, default ""
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    copied : Int
        Copied Int value.
    """
    if isinstance(integer, int):
        copied: Int = Int(integer, variable_name_suffix=variable_name_suffix)
    else:
        copied = integer.copy()
    return copied


@add_debug_info_setting(module_name=__name__)
def get_copied_number_from_builtin_val(
    *, float_or_num: Union[float, Number], variable_name_suffix: str = ""
) -> Number:
    """
    Get a copied number value from a Python built-in float.

    Parameters
    ----------
    float_or_num : float or Number
        Target float (or Number) value.
    variable_name_suffix : str, default ""
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    copied : Number
        Copied Number value.
    """
    if isinstance(float_or_num, (int, float)):
        copied: Number = Number(float_or_num, variable_name_suffix=variable_name_suffix)
    else:
        copied = float_or_num.copy()
    return copied


@add_debug_info_setting(module_name=__name__)
def get_copied_string_from_builtin_val(
    *, string: Union[str, String], variable_name_suffix: str = ""
) -> String:
    """
    Get a copied String value from a Python built-in str.

    Parameters
    ----------
    string : str or String
        Target string value.
    variable_name_suffix : str, default ""
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    copied : String
        Copied String value.
    """

    if isinstance(string, str):
        copied: String = String(string, variable_name_suffix=variable_name_suffix)
    else:
        copied = string.copy()
    return copied


@add_debug_info_setting(module_name=__name__)
def get_copied_boolean_from_builtin_val(
    *, bool_val: Union[bool, Boolean], variable_name_suffix: str = ""
) -> Boolean:
    """
    Get a copied Boolean value from a Python built-in bool.

    Parameters
    ----------
    bool_val : bool or Boolean
        Target bool value.
    variable_name_suffix : str, default ""
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    copied : Boolean
        Copied Boolean value.
    """

    if isinstance(bool_val, bool):
        copied: Boolean = Boolean(bool_val, variable_name_suffix=variable_name_suffix)
    else:
        copied = bool_val.copy()
    return copied



"""The class implementation for the `ballot` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBallotIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `ballot` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" fill-rule="evenodd" height="24" width="24"/><path d="M13,9.5h5v-2h-5V9.5z M13,16.5h5v-2h-5V16.5z M19,21H5c-1.1,0-2-0.9-2-2V5 c0-1.1,0.9-2,2-2h14c1.1,0,2,0.9,2,2v14C21,20.1,20.1,21,19,21z M6,11h5V6H6V11z M7,7h3v3H7V7z M6,18h5v-5H6V18z M7,14h3v3H7V14z" fill-rule="evenodd"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: import_conventions.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# import conventions": "# import の慣習",
    ##################################################
    "The `apysc` library recommends using the `import apysc as ap` to import the package, like the `import numpy as np`, `import pandas as pd`, `import tkinter as tk`, or something like that.": "apyscのライブラリでは他のPythonパッケージの`import numpy as np`、`import pandas as pd`、`import tkinter as tk`などと同じように`import apysc as ap`というimportの指定を推奨しています。",  # noqa
    ##################################################
    "The root package module packages all interfaces you need (for instance, `Sprite`, `Int`, `Stage`).": "モジュールのルートのパッケージパスにapyscで利用が必要な各インターフェイスが設定されています（例 : `Sprite`、`Int`、`Stage`など）。",  # noqa
    ##################################################
    "Also, it packages internal logic modules with the underscore prefix, like the `_file`.": "また、パッケージの内部で使用するロジックの各モジュールは`_file`のようにアンダースコアのプレフィックスが設定されています。",  # noqa
    ##################################################
    "These packages are unnecessary for you.": "基本的にこれらのプレフィックスの付いたパッケージの利用は不要です。",
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = int_1 + ap.Number(10.5)\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = int_1 + ap.Number(10.5)\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: animation_base_start.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# AnimationBase class start interface": "# AnimationBaseクラス start インターフェイス",
    ##################################################
    "This page explains the `AnimationBase` class `start` method interface.": "このページでの`AnimationBase`クラスの`start`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `start` method interface starts the target animation. Each animation interface returns the `AnimationBase` subclass instance is when you call each animation interface, such as the `animation_move` or `animation_x`\\, and it has the `start` method.": "`start`メソッドは対象のアニメーションを開始します。各アニメーションのインターフェイスは`AnimationBase`クラスのサブクラスのインスタンスを返却します。例えば`animation_move`や`animation_x`などが該当し、それらのインスタンスはこの`start`メソッドを持っています。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Notes: you need to call the `start` method to start an animation after the calling of the animation method, such as the `animation_x`\\, as follows:": "特記事項: 以下のコードのようにアニメーションを開始するには`animation_x`などの各アニメーションのインターフェイスを呼び出した後に`start`メソッドを呼ぶ必要があります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100, duration=3000, delay=3000)\nanimation_x.start()\n\nap.save_overall_html(dest_dir_path="./animation_base_start_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100, duration=3000, delay=3000)\nanimation_x.start()\n\nap.save_overall_html(dest_dir_path="./animation_base_start_basic_usage_1/")\n```',  # noqa
    ##################################################
    "You can also use the method chain for code simplicity:": "シンプルに書くためにメソッドチェーンを使う形でも書くことができます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nrectangle.animation_x(x=100, duration=3000, delay=3000).start()\n\nap.save_overall_html(dest_dir_path="./animation_base_start_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nrectangle.animation_x(x=100, duration=3000, delay=3000).start()\n\nap.save_overall_html(dest_dir_path="./animation_base_start_basic_usage_2/")\n```',  # noqa
    ##################################################
    "## start API": "## start API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Start an animation with current settings.<hr>": "現在の設定を使ってアニメーションを開始します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `self`: AnimatonBase": "- `self`: AnimatonBase",
    ##################################################
    "  - This instance.": "  - このインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(x=100).start()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(x=100).start()\n```',  # noqa
}



"""The class implementation for the `not_accessible` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialNotAccessibleIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `not_accessible` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M14,11.05l-3.42-3.42c0.32-0.34,0.74-0.57,1.23-0.61c0.48-0.04,0.84,0.07,1.2,0.26c0.19,0.1,0.39,0.22,0.63,0.46l1.29,1.43 c0.98,1.08,2.53,1.85,4.07,1.83v2C17.25,12.99,15.29,12.12,14,11.05z M12,6c1.1,0,2-0.9,2-2s-0.9-2-2-2c-1.1,0-2,0.9-2,2 S10.9,6,12,6z M2.81,2.81L1.39,4.22L10,12.83V15c0,1.1,0.9,2,2,2h2.17l5.61,5.61l1.41-1.41L2.81,2.81z M10,20c-1.66,0-3-1.34-3-3 c0-1.31,0.84-2.41,2-2.83V12.1c-2.28,0.46-4,2.48-4,4.9c0,2.76,2.24,5,5,5c2.42,0,4.44-1.72,4.9-4h-2.07 C12.42,19.16,11.31,20,10,20z"/></g></svg>'  # noqa



"""This module is for the abstract base class implementation
for if, else if, and else.
"""

from abc import ABC
from abc import abstractmethod
from typing import Any
from typing import Dict
from typing import Optional
from typing import Type

from typing_extensions import final

from apysc._expression.indent_num import Indent
from apysc._type.boolean import Boolean


class IfBase(ABC):
    _condition: Optional[Boolean]
    _locals: Dict[str, Any]
    _globals: Dict[str, Any]
    _snapshot_name: str
    _indent: Indent

    def __init__(
        self,
        *,
        condition: Optional[Boolean],
        locals_: Optional[Dict[str, Any]] = None,
        globals_: Optional[Dict[str, Any]] = None
    ) -> None:
        """
        A class to append if (else if and else) branch instruction
        expression.

        Parameters
        ----------
        condition : Boolean or None
            Boolean value to be used for judgment.
        locals_ : dict or None, default None
            Current scope's local variables. Set locals() value
            to this argument. If specified, this interface
            reverts all local scope VariableNameMixIn
            variables (like Int, Sprite) at the end of the
            `If` or the other branch scope. This setting is
            useful when you don't want to update each variable
            by implementing the `If` or the other scope.
        globals_ : dict or None, default None
            Current scope's global variables. Set the globals()
            value to this argument. This setting works the same
            way as the `locals_` argument.

        References
        ----------
        - If
            - https://simon-ritchie.github.io/apysc/en/if.html
        - Elif
            - https://simon-ritchie.github.io/apysc/en/elif.html
        - Else
            - https://simon-ritchie.github.io/apysc/en/else.html
        - Each branch instruction class's scope variables reverting setting
            - https://simon-ritchie.github.io/apysc/en/branch_instruction_variables_reverting_setting.html  # noqa
        """
        if locals_ is None:
            locals_ = {}
        if globals_ is None:
            globals_ = {}
        self._condition = condition
        self._locals = locals_
        self._globals = globals_
        self._indent = Indent()

    @final
    def __enter__(self) -> Any:
        """
        Method to be called when beginning of the with statement.

        Returns
        -------
        self : IfBase
            This instance.
        """
        from apysc._type import revert_mixin

        self._snapshot_name = revert_mixin.make_snapshots_of_each_scope_vars(
            locals_=self._locals, globals_=self._globals
        )
        self._append_enter_expression()
        self._indent.__enter__()
        return self

    @abstractmethod
    def _append_enter_expression(self) -> None:
        """
        Append branch instruction start expression.
        """

    @final
    def __exit__(self, exc_type: Type, exc_value: Any, traceback: Any) -> None:
        """
        Method to be called when ending of the with statement.

        Parameters
        ----------
        exc_type : Type
            Exception type.
        exc_value : *
            Exception value.
        traceback : *
            Traceback value.
        """
        from apysc._type import revert_mixin

        revert_mixin.revert_each_scope_vars(
            snapshot_name=self._snapshot_name,
            locals_=self._locals,
            globals_=self._globals,
        )
        self._indent.__exit__()
        self._append_exit_expression()
        self._set_last_scope()

    @final
    def _append_exit_expression(self) -> None:
        """
        Append if branch instruction ending expression.
        """
        from apysc._expression import expression_data_util

        expression_data_util.append_js_expression(expression="}")

    @abstractmethod
    def _set_last_scope(self) -> None:
        """
        Set expression last scope value.
        """

    @final
    def _last_scope_is_if_or_elif(self) -> bool:
        """
        Get a boolean value whether the last scope is If or Elif.

        Returns
        -------
        result : bool
            If the last scope is If or Else, this interface
            returns True.
        """
        from apysc._expression import last_scope
        from apysc._expression.last_scope import LastScope

        last_scope_: LastScope = last_scope.get_last_scope()
        if last_scope_ != LastScope.IF and last_scope_ != LastScope.ELIF:
            return False
        return True



"""The mix-in class implementation for the CSS italic (font-style) property.
"""

from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.boolean import Boolean
from apysc._validation import arg_validation_decos


class TextItalicCssMixIn(
    AttrLinkingMixIn,
):
    _italic: Boolean

    def _initialize_italic(self) -> None:
        """
        Initialize the `_italic` attribute.
        """
        if hasattr(self, "_italic"):
            return
        self._italic = Boolean(False)

    @property
    def italic(self) -> Boolean:
        """
        Get an italic (font-style) value.

        Returns
        -------
        italic : Boolean
            An italic (font-style) value.

        References
        ----------
        - Text italic property
            - https://simon-ritchie.github.io/apysc/en/text_italic.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=350,
        ...     stage_height=170,
        ...     stage_elem_id="stage",
        ... )
        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=300,
        ...     font_size=16,
        ...     fill_color=ap.Color("#00aaff"),
        ...     x=25,
        ...     y=25,
        ... )
        >>> multi_line_text.italic = ap.True_
        >>> multi_line_text.italic
        Boolean(True)
        """
        self._initialize_italic()
        return self._italic.copy()

    @italic.setter
    @arg_validation_decos.is_apysc_boolean(arg_position_index=1)
    def italic(self, value: Boolean) -> None:
        """
        Set an italic (font-style) value.

        Parameters
        ----------
        value : Boolean
            An italic (font-style) value.

        References
        ----------
        - Text italic property
            - https://simon-ritchie.github.io/apysc/en/text_italic.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=350,
        ...     stage_height=170,
        ...     stage_elem_id="stage",
        ... )
        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=300,
        ...     font_size=16,
        ...     fill_color=ap.Color("#00aaff"),
        ...     x=25,
        ...     y=25,
        ... )
        >>> multi_line_text.italic = ap.True_
        >>> multi_line_text.italic
        Boolean(True)
        """
        from apysc._branch._else import Else
        from apysc._branch._if import If
        from apysc._display.css_interface import CssInterface
        from apysc._validation import display_validation

        self._initialize_italic()
        interface: CssInterface = display_validation.validate_css_interface(
            instance=self
        )
        self._italic = value

        with If(self._italic):
            interface.set_css(
                name="font-style",
                value="italic",
            )
        with Else():
            interface.set_css(
                name="font-style",
                value="normal",
            )

        self._append_applying_new_attr_val_exp(
            new_attr=self._italic, attr_name="italic"
        )
        self._append_attr_to_linking_stack(attr=self._italic, attr_name="italic")



"""The class implementation for the outlined `disabled_by_default` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialDisabledByDefaultOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `disabled_by_default` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M19,19H5V5h14V19z M3,3v18h18V3H3z M17,15.59L15.59,17L12,13.41L8.41,17L7,15.59L10.59,12L7,8.41L8.41,7L12,10.59L15.59,7 L17,8.41L13.41,12L17,15.59z"/></svg>'  # noqa



"""The class implementation for the outlined `android` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAndroidOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `android` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M6 18c0 .55.45 1 1 1h1v3.5c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5V19h2v3.5c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5V19h1c.55 0 1-.45 1-1V8H6v10zM3.5 8C2.67 8 2 8.67 2 9.5v7c0 .83.67 1.5 1.5 1.5S5 17.33 5 16.5v-7C5 8.67 4.33 8 3.5 8zm17 0c-.83 0-1.5.67-1.5 1.5v7c0 .83.67 1.5 1.5 1.5s1.5-.67 1.5-1.5v-7c0-.83-.67-1.5-1.5-1.5zm-4.97-5.84l1.3-1.3c.2-.2.2-.51 0-.71s-.51-.2-.71 0l-1.48 1.48C13.85 1.23 12.95 1 12 1c-.96 0-1.86.23-2.66.63L7.85.15c-.2-.2-.51-.2-.71 0-.2.2-.2.51 0 .71l1.31 1.31C6.97 3.26 6 5.01 6 7h12c0-1.99-.97-3.75-2.47-4.84zM10 5H9V4h1v1zm5 0h-1V4h1v1z"/></svg>'  # noqa



"""The class implementation for the `youtube_searched_for` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialYoutubeSearchedForIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `youtube_searched_for` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0zm0 0h24v24H0V0z" fill="none"/><path d="M17.01 14h-.8l-.27-.27c.98-1.14 1.57-2.61 1.57-4.23 0-3.59-2.91-6.5-6.5-6.5s-6.5 3-6.5 6.5H2l3.84 4 4.16-4H6.51C6.51 7 8.53 5 11.01 5s4.5 2.01 4.5 4.5c0 2.48-2.02 4.5-4.5 4.5-.65 0-1.26-.14-1.82-.38L7.71 15.1c.97.57 2.09.9 3.3.9 1.61 0 3.08-.59 4.22-1.57l.27.27v.79l5.01 4.99L22 19l-4.99-5z"/></svg>'  # noqa



"""Class implementation for graphic's base class.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._color.color import Color
from apysc._display.display_object import DisplayObject
from apysc._display.line_caps import LineCaps
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_joints import LineJoints
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class GraphicsBase(
    DisplayObject,
):
    _variable_name: str

    @arg_validation_decos.not_empty_string(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def __init__(self, *, variable_name: str) -> None:
        """
        Vector graphic base class.

        Parameters
        ----------
        variable_name : str
            Variable name of this instance. This will be used to
            js expression.
        """
        super(GraphicsBase, self).__init__(variable_name=variable_name)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_basic_values(
        self,
        *,
        fill_color: Color,
        fill_alpha: Union[float, Number],
        line_color: Color,
        line_thickness: Union[int, Int],
        line_alpha: Union[float, Number],
        line_cap: Optional[Union[String, LineCaps]],
        line_joints: Optional[Union[String, LineJoints]]
    ) -> None:
        """
        Set initial fundamental values (such as the fill color or
        line thickness).

        Parameters
        ----------
        fill_color : Color
            A fill-color value to set.
        fill_alpha : float or Number
            A fill-alpha value to set.
        line_color : Color
            A line-color value to set.
        line_thickness : int or Int
            A line-thickness value to set.
        line_alpha : float or Number
            A line-alpha value to set.
        line_cap : String or LineCaps or None
            A line-cap value to set.
        line_joints : String or LineJoints or None
            A line-joints value to set.
        """
        from apysc._display.fill_alpha_mixin import FillAlphaMixIn
        from apysc._display.fill_color_mixin import FillColorMixIn
        from apysc._display.line_alpha_mixin import LineAlphaMixIn
        from apysc._display.line_cap_mixin import LineCapMixIn
        from apysc._display.line_color_mixin import LineColorMixIn
        from apysc._display.line_joints_mixin import LineJointsMixIn
        from apysc._display.line_thickness_mixin import LineThicknessMixIn

        fill_color = fill_color.copy()
        line_color = line_color.copy()

        if isinstance(self, FillColorMixIn):
            self._set_initial_fill_color_if_not_colorless(fill_color=fill_color)
        if isinstance(self, FillAlphaMixIn):
            self._update_fill_alpha_and_skip_appending_exp(value=fill_alpha)
        if isinstance(self, LineColorMixIn):
            self._set_initial_line_color_if_not_colorless(line_color=line_color)
        if isinstance(self, LineThicknessMixIn):
            self._update_line_thickness_and_skip_appending_exp(value=line_thickness)
        if isinstance(self, LineAlphaMixIn):
            self._update_line_alpha_and_skip_appending_exp(value=line_alpha)
        if isinstance(self, LineCapMixIn) and line_cap is not None:
            self._update_line_cap_and_skip_appending_exp(value=line_cap)
        if isinstance(self, LineJointsMixIn) and line_joints is not None:
            self._update_line_joints_and_skip_appending_exp(value=line_joints)

        if isinstance(self, FillAlphaMixIn):
            self._append_applying_new_attr_val_exp(
                new_attr=self._fill_alpha, attr_name="fill_alpha"
            )
            self._append_attr_to_linking_stack(
                attr=self._fill_alpha, attr_name="fill_alpha"
            )

        if isinstance(self, FillColorMixIn):
            self._append_applying_new_attr_val_exp(
                new_attr=self._fill_color, attr_name="fill_color"
            )
            self._append_attr_to_linking_stack(
                attr=self._fill_color, attr_name="fill_color"
            )

        if isinstance(self, LineColorMixIn):
            self._append_applying_new_attr_val_exp(
                new_attr=self._line_color, attr_name="line_color"
            )
            self._append_attr_to_linking_stack(
                attr=self._line_color, attr_name="line_color"
            )

        if isinstance(self, LineAlphaMixIn):
            self._append_applying_new_attr_val_exp(
                new_attr=self._line_alpha, attr_name="line_alpha"
            )
            self._append_attr_to_linking_stack(
                attr=self._line_alpha, attr_name="line_alpha"
            )

        if isinstance(self, LineThicknessMixIn):
            self._append_applying_new_attr_val_exp(
                new_attr=self._line_thickness, attr_name="line_thickness"
            )
            self._append_attr_to_linking_stack(
                attr=self._line_thickness, attr_name="line_thickness"
            )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_line_setting_if_not_none_value_exists(
        self,
        line_dot_setting: Optional[LineDotSetting],
        line_dash_setting: Optional[LineDashSetting],
        line_round_dot_setting: Optional[LineRoundDotSetting],
        line_dash_dot_setting: Optional[LineDashDotSetting],
    ) -> None:
        """
        If a line setting (dot, dash, or something else) with a value
        other than None exists, set that value to the attribute.

        Parameters
        ----------
        line_dot_setting : Optional[LineDotSetting]
            A dot setting to set.
        line_dash_setting : Optional[LineDashSetting]
            A dash setting to set.
        line_round_dot_setting : Optional[LineRoundDotSetting]
            A round-dot setting to set.
        line_dash_dot_setting : Optional[LineDashDotSetting]
            A dash-dot (1-dot chain) setting to set.
        """
        if line_dot_setting is not None:
            self.line_dot_setting = line_dot_setting
            return
        if line_dash_setting is not None:
            self.line_dash_setting = line_dash_setting
            return
        if line_round_dot_setting is not None:
            self.line_round_dot_setting = line_round_dot_setting
            return
        if line_dash_dot_setting is not None:
            self.line_dash_dot_setting = line_dash_dot_setting
            return



"""This module is for the translation mapping data of the
following document:

Document file: assert_dicts_equal_and_dicts_not_equal.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# assert_dicts_equal and assert_dicts_not_equal interfaces": "# assert_dicts_equal と assert_dicts_not_equal インターフェイス",  # noqa
    ##################################################
    "This page explains the `assert_dicts_equal` and `assert_dicts_not_equal` function interfaces.": "このページでは`assert_dicts_equal`と`assert_dicts_not_equal`の各関数のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `assert_dicts_equal` function interface asserts specified two dictionaries (`dict` or `Dictionary` type value) are equal. Conversely, the `assert_dicts_not_equal` function interface asserts specified two dictionaries are not equal.": "`assert_dicts_equal`関数のインターフェイスは指定された2つの辞書（`Dictionary`型など）の値が一致しているかをチェックします。逆に`assert_dicts_not_equal`関数のインターフェイスは指定された2つの辞書の値が一致していないことをチェックします。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [JavaScript assertion interface basic behavior](assertion_basic_behavior.md)": "- [JavaScriptの各アサーションのインターフェイスの基本的な挙動](jp_assertion_basic_behavior.md)",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Both of the `assert_dicts_equal` and `assert_dicts_not_equal` interfaces require the `left` and `right` arguments. The `msg` argument is optional.": "`assert_dicts_equal`と`assert_dicts_not_equal`の各インターフェイスは`left`と`right`引数を必要とします。`msg`引数は省略可です。",  # noqa
    ##################################################
    "You can specify a Python built-in `dict` and `Dictionary` values to these arguments.": "各インターフェイスにはPythonビルトインの`dict`やapyscの`Dictionary`の値を引数として指定することができます。",  # noqa
    ##################################################
    "The following example (`assert_dicts_equal` and values are equal) passes:": "以下の例では`assert_dicts_equal`関数を使って値が同じ辞書に対してチェックを行っています（チェックは通ります）。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 10, "b": 20})\nap.assert_dicts_equal(\n    left={"a": 10, "b": 20}, right=dict_val, msg="Values are not equal!"\n)\n\nap.save_overall_html(dest_dir_path="assert_dicts_equal_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 10, "b": 20})\nap.assert_dicts_equal(\n    left={"a": 10, "b": 20}, right=dict_val, msg="Values are not equal!"\n)\n\nap.save_overall_html(dest_dir_path="assert_dicts_equal_basic_usage_1/")\n```',  # noqa
    ##################################################
    "```\n[assert_dicts_equal]\nLeft value: {a: 10, b: 20} right value: dct_1\n```": "```\n[assert_dicts_equal]\nLeft value: {a: 10, b: 20} right value: dct_1\n```",  # noqa
    ##################################################
    "The following example (`assert_dicts_equal` and values are not equal)  fails:": "以下の例では`assert_dicts_equal`関数を使って値が一致していない辞書に対してチェックを行っています（チェックは失敗します）。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 10, "b": 20})\nap.assert_dicts_equal(left={"a": 30}, right=dict_val, msg="Values are not equal!")\n\nap.save_overall_html(dest_dir_path="assert_dicts_equal_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 10, "b": 20})\nap.assert_dicts_equal(left={"a": 30}, right=dict_val, msg="Values are not equal!")\n\nap.save_overall_html(dest_dir_path="assert_dicts_equal_basic_usage_2/")\n```',  # noqa
    ##################################################
    "```\n[assert_dicts_equal]\nLeft value: {a: 30} right value: dct_1\n...\nAssertion failed: Values are not equal!\n```": "```\n[assert_dicts_equal]\nLeft value: {a: 30} right value: dct_1\n...\nAssertion failed: Values are not equal!\n```",  # noqa
    ##################################################
    "The following example (`assert_dicts_not_equal` and values are not equal) passes:": "以下の例では`assert_dicts_not_equal`関数を使って値が一致していない辞書に対してチェックを行っています（チェックは通ります）。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 10, "b": 20})\nap.assert_dicts_not_equal(left={"a": 30}, right=dict_val, msg="Values are equal!")\n\nap.save_overall_html(dest_dir_path="assert_dicts_not_equal_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 10, "b": 20})\nap.assert_dicts_not_equal(left={"a": 30}, right=dict_val, msg="Values are equal!")\n\nap.save_overall_html(dest_dir_path="assert_dicts_not_equal_basic_usage_1/")\n```',  # noqa
    ##################################################
    "```\n[assert_dicts_not_equal]\nLeft value: {a: 30} right value: dct_1\n```": "```\n[assert_dicts_not_equal]\nLeft value: {a: 30} right value: dct_1\n```",  # noqa
    ##################################################
    "## Notes for the assert_equal and assert_not_equal interfaces": "## assert_equal と assert_not_equal の各インターフェイスにおける特記事項",  # noqa
    ##################################################
    "If a `Dictionary` value is specified to the `assert_equal` or `assert_not_equal` interface's argument value, then the `assert_dicts_equal` or `assert_dicts_not_equal` interfaces will be called instead of the `assert_equal` or `assert_not_equal` interfaces automatically.": "もし`assert_equal`もしくは`assert_not_equal`のインターフェイスの引数へ`Dictionary`の値が指定された場合、自動的に`assert_dicts_equal`もしくは`assert_dicts_not_equal`のインターフェイスがそれらのインターフェイスの代わりに呼ばれます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 30})\nap.assert_equal(left={"a": 30}, right=dict_val, msg="Values are not equal!")\n\nap.save_overall_html(dest_dir_path="assert_dicts_equal_notes_for_assert_equal/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 30})\nap.assert_equal(left={"a": 30}, right=dict_val, msg="Values are not equal!")\n\nap.save_overall_html(dest_dir_path="assert_dicts_equal_notes_for_assert_equal/")\n```',  # noqa
    ##################################################
    "```\n[assert_dicts_equal]\nLeft value: {a: 30} right value: dct_1\n```": "```\n[assert_dicts_equal]\nLeft value: {a: 30} right value: dct_1\n```",  # noqa
    ##################################################
    "## assert_dicts_equal API": "## assert_dicts_equal API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the Dictionary values equal condition.<hr>": "辞書の値の等値条件のためのJavaScript上のアサーションのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `left`: *": "- `left`: *",
    ##################################################
    "  - Left-side value to compare.": "  - 比較用の左辺の値。",
    ##################################################
    "- `right`: *": "- `right`: *",
    ##################################################
    "  - Right-side value to compare.": "  - 比較用の右辺の値。",
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "- `outer_frames_index_adjustment`: int, optional": "- `outer_frames_index_adjustment`: int, optional",  # noqa
    ##################################################
    "  - The trace's outer frames index adjustment setting. This function uses this argument to adjust the caller's information. Also, this function only uses this argument in internal logic.": "  - trace関数の関数外の参照するフレームのインデックスの調整値です。この引数は呼び出し元の情報の位置を調整するのに使用されます。また、この引数は内部のロジックでのみ使用されるため通常は設定する必要はありません。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    'This interface is used instead of assert_equal for Dictionary class comparison (JavaScript can not compare dictionary (Object) directly, like a Python, for example, `{"a": 10} === {"a": 10}` becomes false).<hr>': "このインターフェイスは`assert_equal`関数での`Dictionary`のクラスの値の比較時には代わりに使用されます（JavaScriptではPythonと異なり辞書の値を直接比較できないため）。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> dict_1: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> dict_2: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> ap.assert_dicts_equal(dict_1, dict_2)\n```': '```py\n>>> import apysc as ap\n>>> dict_1: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> dict_2: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> ap.assert_dicts_equal(dict_1, dict_2)\n```',  # noqa
    ##################################################
    "## assert_dicts_not_equal API": "## assert_dicts_not_equal API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the Dictionary values not equal condition.<hr>": "辞書の値の非等値条件のためのJavaScript上のアサーションのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `left`: *": "- `left`: *",
    ##################################################
    "  - Left-side value to compare.": "  - 比較用の左辺の値。",
    ##################################################
    "- `right`: *": "- `right`: *",
    ##################################################
    "  - Right-side value to compare.": "  - 比較用の右辺の値。",
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "- `outer_frames_index_adjustment`: int, optional": "- `outer_frames_index_adjustment`: int, optional",  # noqa
    ##################################################
    "  - The trace's outer frames index adjustment setting. This function uses this argument to adjust the caller's information. Also, this function only uses this argument in internal logic.": "  - trace関数の関数外の参照するフレームのインデックスの調整値です。この引数は呼び出し元の情報の位置を調整するのに使用されます。また、この引数は内部のロジックでのみ使用されるため通常は設定する必要はありません。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    'This interface is used instead of assert_not_equal for Dictionary class comparison (JavaScript can not compare dictionary (Object) directly, like a Python, for example, `{"a": 10} !== {"a": 10}` becomes true).<hr>': "このインターフェイスは`assert_not_equal`関数での`Dictionary`クラスの値の比較時には代わりに使用されます（JavaScriptではPythonと異なり辞書の値を直接比較できないため）。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> dict_1: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> dict_2: ap.Dictionary = ap.Dictionary({"a": 20})\n>>> ap.assert_dicts_not_equal(dict_1, dict_2)\n```': '```py\n>>> import apysc as ap\n>>> dict_1: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> dict_2: ap.Dictionary = ap.Dictionary({"a": 20})\n>>> ap.assert_dicts_not_equal(dict_1, dict_2)\n```',  # noqa
}



"""The class implementation for the `settings_ethernet` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSettingsEthernetIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `settings_ethernet` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M7.77 6.76L6.23 5.48.82 12l5.41 6.52 1.54-1.28L3.42 12l4.35-5.24zM7 13h2v-2H7v2zm10-2h-2v2h2v-2zm-6 2h2v-2h-2v2zm6.77-7.52l-1.54 1.28L20.58 12l-4.35 5.24 1.54 1.28L23.18 12l-5.41-6.52z"/></svg>'  # noqa



"""The class implementation for the `person_add_disabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPersonAddDisabledIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `person_add_disabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><circle cx="15" cy="8" r="4"/><path d="M23 20v-2c0-2.3-4.1-3.7-6.9-3.9l6 5.9h.9zm-11.6-5.5C9.2 15.1 7 16.3 7 18v2h9.9l4 4 1.3-1.3-21-20.9L0 3.1l4 4V10H1v2h3v3h2v-3h2.9l2.5 2.5zM6 10v-.9l.9.9H6z"/></svg>'  # noqa



"""The class implementation for the `av_timer` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAvTimerIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `av_timer` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M11 17c0 .55.45 1 1 1s1-.45 1-1-.45-1-1-1-1 .45-1 1zm0-14v4h2V5.08c3.39.49 6 3.39 6 6.92 0 3.87-3.13 7-7 7s-7-3.13-7-7c0-1.68.59-3.22 1.58-4.42L12 13l1.41-1.41-6.8-6.8v.02C4.42 6.45 3 9.05 3 12c0 4.97 4.02 9 9 9 4.97 0 9-4.03 9-9s-4.03-9-9-9h-1zm7 9c0-.55-.45-1-1-1s-1 .45-1 1 .45 1 1 1 1-.45 1-1zM6 12c0 .55.45 1 1 1s1-.45 1-1-.45-1-1-1-1 .45-1 1z"/></svg>'  # noqa
